import unittest

from ..crud.base import CRUDBase
from .models import Sample
from .schemas import SampleCreate, SampleUpdate
from .crud import samples


class CRUDSampleError(CRUDBase[Sample, SampleCreate, SampleUpdate]):
    ...


class TestCRUD(unittest.TestCase):
    def test_no_model(self) -> None:

        with self.assertRaises(TypeError):
            CRUDSampleError()

    def test_model(self) -> None:
        self.assertTrue(hasattr(samples, "model"))
        self.assertEqual(samples.model, Sample)

        # create an instance from property
        sm = samples.model(email="sample@sample")
        self.assertIsInstance(sm, Sample)



from typing import Any
from uuid import uuid1, UUID
from sqlalchemy.orm import DeclarativeBase
from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy.orm import Mapped, mapped_column


class ModelBase(DeclarativeBase):
    id: Mapped[int] = mapped_column(primary_key=True, index=True)
    uid: Mapped[UUID] = mapped_column(index=True, unique=True, default=uuid1)
    __name__: str

    # Generate __tablename__ automatically
    @declared_attr
    def __tablename__(cls) -> str:
        return cls.__name__.lower()



from ..crud import CRUDBase
from .models import Sample as SampleModel
from .schemas import SampleCreate, SampleUpdate



class CRUDSample(CRUDBase[SampleModel, SampleCreate, SampleUpdate]):
    model = SampleModel


samples = CRUDSample()


from sqlalchemy.orm import Mapped, mapped_column
from secrets import token_urlsafe
from ..models import ModelBase, Timestamp


class Sample(Timestamp, ModelBase):
    token: Mapped[str] = mapped_column(default=token_urlsafe)
    describe: Mapped[str] = mapped_column(nullable=True)
    email: Mapped[str] = mapped_column(nullable=True, unique=True)